@@ -1,6 +1,3 @@
-use crate::cli::Mode;
-use crate::config::Config;
-use crate::handle_error;
 // use crate::utils::handle_error_result;
 use console::Style;
 use libmacchina::traits::GeneralReadout as _;
@@ -10,8 +7,13 @@ use libmacchina::GeneralReadout;
 use libmacchina::KernelReadout;
 use libmacchina::PackageReadout;
 use user_error::{UserFacingError, UFE};
+
+use crate::cli::Mode;
+use crate::config::Config;
+use crate::handle_error;
+
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Format {
     pub mode: Mode,
     pub top_left_corner_char: char,
@@ -27,7 +29,7 @@ pub struct Format {
 
 impl Default for Format {
     fn default() -> Self {
-        Format {
+        Self {
             mode: Mode::Classic,
             top_left_corner_char: '╭',
             top_right_corner_char: '╮',
@@ -43,7 +45,7 @@ impl Default for Format {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct User {
     pre_text_style: String,
     pre_text: String,
@@ -54,7 +56,7 @@ pub struct User {
 
 impl Default for User {
     fn default() -> Self {
-        User {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from(""),
             output_style: String::from("bold.yellow"),
@@ -80,7 +82,7 @@ impl User {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Delimiter {
     style: String,
     repeat_num: usize,
@@ -89,7 +91,7 @@ pub struct Delimiter {
 
 impl Default for Delimiter {
     fn default() -> Self {
-        Delimiter {
+        Self {
             style: String::from("white"),
             repeat_num: 0,
             char: '-',
@@ -111,7 +113,7 @@ impl Delimiter {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Os {
     pre_text_style: String,
     pre_text: String,
@@ -120,7 +122,7 @@ pub struct Os {
 
 impl Default for Os {
     fn default() -> Self {
-        Os {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("OS: "),
             output_style: String::from("white"),
@@ -129,7 +131,7 @@ impl Default for Os {
 }
 
 impl Os {
-    pub fn get_os(&self) -> String {
+    pub fn get_os() -> String {
         let general_readout = GeneralReadout::new();
         let os: String;
         if cfg!(target_os = "linux") {
@@ -140,9 +142,9 @@ impl Os {
         os
     }
     pub fn get_info(&self) -> String {
-        let os = self.get_os();
+        let os = Os::get_os();
         let build_version = Config::run_cmd("sw_vers -buildVersion", "Failed to get build version");
-        let arch = Config::run_cmd("uname -m", "Failed to get arch");
+        let arch = Config::run_cmd("machine", "Failed to get arch");
 
         let output_style = Style::from_dotted_str(&self.output_style);
         format!(
@@ -156,7 +158,7 @@ impl Os {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Host {
     pre_text_style: String,
     pre_text: String,
@@ -165,7 +167,7 @@ pub struct Host {
 
 impl Default for Host {
     fn default() -> Self {
-        Host {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Host: "),
             output_style: String::from("white"),
@@ -184,8 +186,9 @@ impl Host {
         )
     }
 }
+
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Kernel {
     pre_text_style: String,
     pre_text: String,
@@ -194,7 +197,7 @@ pub struct Kernel {
 
 impl Default for Kernel {
     fn default() -> Self {
-        Kernel {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Kernel: "),
             output_style: String::from("white"),
@@ -218,7 +221,7 @@ impl Kernel {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Uptime {
     pre_text_style: String,
     pre_text: String,
@@ -228,7 +231,7 @@ pub struct Uptime {
 
 impl Default for Uptime {
     fn default() -> Self {
-        Uptime {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Uptime: "),
             output_style: String::from("white"),
@@ -260,8 +263,9 @@ impl Uptime {
         )
     }
 }
+
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Packages {
     pre_text_style: String,
     pre_text: String,
@@ -270,7 +274,7 @@ pub struct Packages {
 
 impl Default for Packages {
     fn default() -> Self {
-        Packages {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Packages: "),
             output_style: String::from("white"),
@@ -295,7 +299,7 @@ impl Packages {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Shell {
     pre_text_style: String,
     pre_text: String,
@@ -304,7 +308,7 @@ pub struct Shell {
 
 impl Default for Shell {
     fn default() -> Self {
-        Shell {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Shell: "),
             output_style: String::from("white"),
@@ -338,7 +342,7 @@ impl Shell {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Resolution {
     pre_text_style: String,
     pre_text: String,
@@ -347,7 +351,7 @@ pub struct Resolution {
 
 impl Default for Resolution {
     fn default() -> Self {
-        Resolution {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Resolution: "),
             output_style: String::from("white"),
@@ -369,7 +373,7 @@ impl Resolution {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct DesktopEnvironment {
     pre_text_style: String,
     pre_text: String,
@@ -378,7 +382,7 @@ pub struct DesktopEnvironment {
 
 impl Default for DesktopEnvironment {
     fn default() -> Self {
-        DesktopEnvironment {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Desktop Environment: "),
             output_style: String::from("white"),
@@ -403,7 +407,7 @@ impl DesktopEnvironment {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct WindowManager {
     pre_text_style: String,
     pre_text: String,
@@ -412,7 +416,7 @@ pub struct WindowManager {
 
 impl Default for WindowManager {
     fn default() -> Self {
-        WindowManager {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Window Manager: "),
             output_style: String::from("white"),
@@ -437,7 +441,7 @@ impl WindowManager {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Terminal {
     pre_text_style: String,
     pre_text: String,
@@ -446,7 +450,7 @@ pub struct Terminal {
 
 impl Default for Terminal {
     fn default() -> Self {
-        Terminal {
+        Self {
             pre_text_style: String::from("bold.yellow"),
             pre_text: String::from("Terminal: "),
             output_style: String::from("white"),
@@ -468,7 +472,38 @@ impl Terminal {
 }
 
 #[derive(Deserialize, Debug, PartialEq, Clone)]
-#[serde(deny_unknown_fields, default)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
+pub struct Cpu {
+    pre_text_style: String,
+    pre_text: String,
+    output_style: String,
+}
+
+impl Default for Cpu {
+    fn default() -> Self {
+        Self {
+            pre_text_style: String::from("bold.yellow"),
+            pre_text: String::from("CPU: "),
+            output_style: String::from("white"),
+        }
+    }
+}
+
+impl Cpu {
+    pub fn get_info(&self) -> String {
+        let general_readout = GeneralReadout::new();
+        let cpu = handle_error!(general_readout.cpu_model_name(), "Failed to get CPU name");
+
+        format!(
+            "{}{}",
+            Style::from_dotted_str(&self.pre_text_style).apply_to(&self.pre_text),
+            Style::from_dotted_str(&self.output_style).apply_to(cpu),
+        )
+    }
+}
+
+#[derive(Deserialize, Debug, PartialEq, Clone)]
+#[serde(deny_unknown_fields, default, rename_all = "kebab-case")]
 pub struct Module {
     pre_text_style: String,
     pre_text: String,
@@ -490,7 +525,7 @@ impl Module {
 
 impl Default for Module {
     fn default() -> Self {
-        Module {
+        Self {
             command: String::from(""),
             output_style: String::from("white"),
             pre_text: String::from(""),
